#ifndef POINT2DF_H
#define POINT2DF_H
#include <math.h>

class  Point2df {
public:
	float x;
	float y;
public:

	/// Default constructor; value is not initialized
	Point2df() {x=0;y=0;}

	/// Initialize from another vector
	Point2df(const Point2df& v) :x(v.x), y(v.y) {}

	/// Initialize from array of floats
	Point2df(const float v[]) :x(v[0]), y(v[1]) {}

	/// Initialize from components
	Point2df(float xx, float yy) :x(xx), y(yy) {}

	/// Return a non-const reference to the ith element
	float& operator[](int i) { return (&x)[i]; }

	/// Return a const reference to the ith element
	const float& operator[](int i) const { return (&x)[i]; }

	/// Assignment
	Point2df& operator=(const Point2df& v) { x = v.x; y = v.y; return *this; }

	/// Assignment from array of floats
	Point2df& operator=(const float v[]) { x = v[0]; y = v[1]; return *this; }

	/// Set value from another vector
	Point2df& set(const Point2df& v) { x = v.x; y = v.y; return *this; }

	/// Set value from array of floats
	Point2df& set(const float v[]) { x = v[0]; y = v[1]; return *this; }

	/// Set value from components
	Point2df& set(float xx, float yy) { x = xx; y = yy; return *this; }

	/// Assigning operators
	Point2df& operator*=(float n) { return set(x*n, y*n); }
	Point2df& operator/=(float n) { return set(x / n, y / n); }
	Point2df& operator+=(const Point2df& v) { return set(x + v.x, y + v.y); }
	Point2df& operator-=(const Point2df& v) { return set(x - v.x, y - v.y); }

	/// Test if zero
	bool operator!() const { return x == 0.0f && y == 0.0f; }

	/// Unary
	Point2df operator+() const { return *this; }
	Point2df operator-() const { return Point2df(-x, -y); }

	/// Length and square of length
	float length2() const { return x*x + y*y; }
	float length() const { return sqrt(length2()); }
    float getX() const;
	float getY() const;

	/// Destructor
	~Point2df() {}
};


/// Dot product
inline float operator*(const Point2df& a, const Point2df& b)
{ return a.x*b.x + a.y*b.y; }

/// Scaling
inline Point2df operator*(const Point2df& a, float n)
{ return Point2df(a.x*n, a.y*n); }
inline Point2df operator*(float n, const Point2df& a)
{ return Point2df(n*a.x, n*a.y); }
inline Point2df operator/(const Point2df& a, float n)
{ return Point2df(a.x / n, a.y / n); }


/// Vector and vector addition
inline Point2df operator+(const Point2df& a, const Point2df& b)
{ return Point2df(a.x + b.x, a.y + b.y); }
inline Point2df operator-(const Point2df& a, const Point2df& b)
{ return Point2df(a.x - b.x, a.y - b.y); }



/// Equality tests
inline bool operator==(const Point2df& a, const Point2df& b)
{ return a.x == b.x && a.y == b.y; }
inline bool operator!=(const Point2df& a, const Point2df& b)
{ return a.x != b.x || a.y != b.y; }


/// Lowest or highest components
inline Point2df lo(const Point2df& a, const Point2df& b)
{ return Point2df(fmin(a.x, b.x), fmin(a.y, b.y)); }
inline Point2df hi(const Point2df& a, const Point2df& b)
{ return Point2df(fmax(a.x, b.x), fmax(a.y, b.y)); }

/// Normalize vector
extern  Point2df normalize(const Point2df& v);


/// Linearly interpolate
extern  Point2df lerp(const Point2df& u, const Point2df& v, float f);
extern Point2df perp(const Point2df &v);

#endif


